@@ -18,8 +18,9 @@
 #
 import os
 import numpy as np
-import pint
 import logging
+import re
+import datetime
 
 import panedr
 try:
@@ -29,11 +30,12 @@ except Exception:
     MDAnalysis = False
 
 from .metainfo import m_env
+from nomad.units import ureg
 from nomad.parsing.parser import FairdiParser
 
 from nomad.parsing.file_parser import TextParser, Quantity, FileParser
 from nomad.datamodel.metainfo.common_dft import Run, SamplingMethod, System,\
-    EnergyContribution, SingleConfigurationCalculation, Topology, Interaction
+    EnergyContribution, SingleConfigurationCalculation, Topology, Interaction, Method
 from .metainfo.gromacs import x_gromacs_section_control_parameters, x_gromacs_section_input_output_files
 
 MOL = 6.022140857e+23
@@ -41,165 +43,101 @@ MOL = 6.022140857e+23
 
 class GromacsLogParser(TextParser):
     def __init__(self):
-        run_control = [
-            'integrator', 'tinit', 'dt', 'nsteps', 'init-step', 'simulation-part',
-            'comm-mode', 'nstcomm', 'comm-grps']
-
-        langevin_dynamics = ['bd-fric', 'ld-seed']
-
-        energy_minimization = ['emtol', 'emstep', 'nstcgsteep', 'nbfgscorr']
-
-        shell_molecular_dynamics = ['niter', 'fcstep']
-
-        test_particle_insertion = ['rtpi']
-
-        output_control = [
-            'nstxout', 'nstvout', 'nstfout', 'nstlog', 'nstcalcenergy', 'nstenergy',
-            'nstxout-compressed', 'compressed-x-precision', 'compressed-x-grps',
-            'energygrps']
-
-        neigbor_searching = [
-            'cutoff-scheme', 'nstlist', 'pbc', 'periodic-molecules', 'verlet-buffer-tolerance',
-            'rlist']
-
-        electrostatics = [
-            'coulombtype', 'coulomb-modifier', 'rcoulomb-switch', 'rcoulomb', 'epsilon-r',
-            'epsilon-rf']
-
-        van_der_waals = ['vdwtype', 'vdw-modifier', 'rvdw-switch', 'rvdw', 'DispCorr']
-
-        tables = ['table-extension', 'energygrp-table']
-
-        ewald = [
-            'fourierspacing', 'fourier-nx', 'fourier-ny', 'fourier-nz', 'pme-order',
-            'ewald-rtol', 'ewald-rtol-lj', 'lj-pme-comb-rule', 'ewald-geometry',
-            'epsilon-surface']
-
-        temperature_coupling = [
-            'tcoupl', 'nsttcouple', 'nh-chain-length', 'print-nose-hoover-chain-variables',
-            'tc-grps', 'tau-t', 'ref-t']
-
-        pressure_coupling = [
-            'pcoupl', 'pcoupltype', 'nstpcouple', 'tau-p', 'compressibility', 'ref-p',
-            'refcoord-scaling']
-
-        simulated_annealing = [
-            'annealing', 'annealing-npoints', 'annealing-time', 'annealing-temp']
-
-        velocity_generation = ['gen-vel', 'gen-temp', 'gen-seed']
-
-        bonds = [
-            'constraints', 'constraint-algorithm', 'continuation', 'shake-tol', 'lincs-order',
-            'lincs-iter', 'lincs-warnangle', 'morse']
-
-        energy_group_exclusions = ['energygrp-excl']
-
-        walls = [
-            'nwall', 'wall-atomtype', 'wall-type', 'table', 'wall-r-linpot', 'wall-density',
-            'wall-ewald-zfac']
-
-        com_pulling = [
-            'pull', 'pull-cylinder-r', 'pull-constr-tol', 'pull-print-com', 'pull-print-ref-value',
-            'pull-print-components', 'pull-nstxout', 'pull-nstfout', 'pull-pbc-ref-prev-step-com',
-            'pull-xout-average', 'pull-ngroups', 'pull-ncoords', 'pull-group1-name',
-            'pull-group1-weights', 'pull-group1-pbcatom', 'pull-coord1-type',
-            'pull-coord1-potential-provider', 'pull-coord1-geometry', 'pull-coord1-groups',
-            'pull-coord1-dim', 'pull-coord1-origin', ' pull-coord1-vec', 'pull-coord1-start',
-            'pull-coord1-init', 'pull-coord1-rate', 'pull-coord1-k', 'pull-coord1-kB']
-
-        awh_adaptive_biasing = [
-            'awh', 'awh-potential', 'awh-share-multisim', 'awh-seed', 'awh-nstout',
-            'awh-nstsample', 'awh-nsamples-update', 'awh-nbias', 'awh1-error-init',
-            'awh1-growth', 'awh1-equilibrate-histogram', 'awh1-target', 'awh1-target-beta-scaling',
-            'awh1-target-cutoff', 'awh1-user-data', 'awh1-share-group', 'awh1-ndim',
-            'awh1-dim1-coord-provider', 'awh1-dim1-coord-index', 'awh1-dim1-force-constant',
-            'awh1-dim1-start', 'awh1-dim1-end', 'awh1-dim1-diffusion', 'awh1-dim1-cover-diameter']
-
-        enforced_rotation = [
-            'rotation', 'rot-ngroups', 'rot-group0', 'rot-type0', 'rot-massw0', 'rot-vec0',
-            'rot-pivot0', 'rot-rate0', 'rot-k0', 'rot-slab-dist0', 'rot-min-gauss0',
-            'rot-eps0', 'rot-fit-method0', 'rot-potfit-nsteps0', 'rot-potfit-step0',
-            'rot-nstrout', 'rot-nstsout']
-
-        nmr_refinement = [
-            'disre', 'disre-weighting', 'disre-mixed', 'disre-fc', 'disre-tau', 'nstdisreout',
-            'orire', 'orire-fc', 'orire-tau', 'orire-fitgrp', 'nstorireout']
-
-        free_energy_calculations = [
-            'free-energy', 'expanded', 'init-lambda', 'delta-lambda', 'init-lambda-state',
-            'fep-lambdas', 'coul-lambdas', 'vdw-lambdas', 'bonded-lambdas', 'restraint-lambdas',
-            'mass-lambdas', 'temperature-lambdas', 'calc-lambda-neighbors', 'sc-alpha',
-            'sc-r-power', 'sc-coul', 'sc-power', 'sc-sigma', 'couple-moltype', 'couple-lambda0',
-            'couple-lambda1', 'couple-intramol', 'nstdhdl', 'dhdl-derivatives',
-            'dhdl-print-energy', 'separate-dhdl-file', 'dh-hist-size', 'dh-hist-spacing']
-
-        expanded_ensemble_calculations = [
-            'nstexpanded', 'lmc-stats', 'lmc-mc-move', 'lmc-seed', 'mc-temperature', 'wl-ratio',
-            'wl-scale', 'init-wl-delta', 'wl-oneovert', 'lmc-repeats', 'lmc-gibbsdelta',
-            'lmc-forced-nstart', 'nst-transition-matrix', 'symmetrized-transition-matrix',
-            'mininum-var-min', 'init-lambda-weights', 'lmc-weights-equil', 'simulated-tempering',
-            'sim-temp-low', 'sim-temp-high', 'simulated-tempering-scaling']
-
-        non_equilibrium_md = [
-            'acc-grps', 'accelerate', 'freezegrps', 'freezedim', 'cos-acceleration',
-            'deform']
-
-        electric_fields = ['electric-field-x', 'electric-field-y', 'electric-field-z']
-
-        mixed_quantum_classical_molecular_dynamics = [
-            'QMMM', 'QMMM-grps', 'QMMMscheme', 'QMmethod', 'QMbasis', 'QMcharge', 'QMmult',
-            'CASorbitals', 'CASelectrons', 'SH']
-
-        computational_electrophysiology = [
-            'swapcoords', 'swap-frequency', 'split-group0', 'split-group1', 'massw-split0',
-            'massw-split1', 'solvent-group', 'coupl-steps', 'iontypes', 'iontype0-name',
-            'iontype0-in-A', 'iontype0-in-B', 'bulk-offsetA', 'bulk-offsetB', 'threshold',
-            'cyl0-r', 'cyl0-up', 'cyl0-down', 'cyl1-r', 'cyl1-up', 'cyl1-down']
-
-        density_guided_simulations = [
-            'density-guided-simulation-active', 'density-guided-simulation-group',
-            'density-guided-simulation-similarity-measure', 'density-guided-simulation-atom-spreading-weight',
-            'density-guided-simulation-force-constant', 'density-guided-simulation-gaussian-transform-spreading-width',
-            'density-guided-simulation-gaussian-transform-spreading-range-in-multiples-of-width',
-            'density-guided-simulation-reference-density-filename', 'density-guided-simulation-nst',
-            'density-guided-simulation-normalize-densities', 'density-guided-simulation-adaptive-force-scaling',
-            'density-guided-simulation-adaptive-force-scaling-time-constant']
-
-        self._commands = ['version']
-
-        self._commands +=\
-            run_control + langevin_dynamics + energy_minimization + shell_molecular_dynamics +\
-            test_particle_insertion + output_control + neigbor_searching + electrostatics +\
-            van_der_waals + tables + ewald + temperature_coupling + pressure_coupling +\
-            simulated_annealing + velocity_generation + bonds + energy_group_exclusions +\
-            walls + com_pulling + awh_adaptive_biasing + enforced_rotation + nmr_refinement +\
-            free_energy_calculations + expanded_ensemble_calculations + non_equilibrium_md +\
-            electric_fields + mixed_quantum_classical_molecular_dynamics + computational_electrophysiology +\
-            density_guided_simulations
-
         super().__init__(None)
 
     def init_quantities(self):
-        def str_op(val):
-            val = val.strip().replace(',', '').replace('{', '').replace('}', '')
-            val = val.split()
-            val = val[0] if len(val) == 1 else val
-            return val
+        def str_to_header(val_in):
+            val = [v.split(':', 1) for v in val_in.strip().split('\n')]
+            return {v[0].strip(): v[1].strip() for v in val if len(v) == 2}
 
-        self._quantities = []
-        for command in self._commands:
-            self._quantities.append(
-                Quantity(
-                    command, r'\s*%s\s*\[*\s*\d*\s*\]*\s*[=:]+([\s\S]*?\n)' % command,
-                    str_operation=str_op, repeats=False)
-            )
+        def str_to_input_parameters(val_in):
+            re_array = re.compile(r'\s*([\w\-]+)\[[\d ]+\]\s*=\s*\{*(.+)')
+            re_scalar = re.compile(r'\s*([\w\-]+)\s*=\s*(.+)')
+            parameters = dict()
+            val = val_in.strip().split('\n')
+            for val_n in val:
+                val_scalar = re_scalar.match(val_n)
+                if val_scalar:
+                    parameters[val_scalar.group(1)] = val_scalar.group(2)
+                    continue
+                val_array = re_array.match(val_n)
+                if val_array:
+                    parameters.setdefault(val_array.group(1), [])
+                    value = [float(v) for v in val_array.group(2).rstrip('}').split(',')]
+                    parameters[val_array.group(1)].append(value[0] if len(value) == 1 else value)
+            return parameters
+
+        def str_to_energies(val_in):
+            energy_keys_re = re.compile(r'(.+?)(?:  |\Z| P)')
+            keys = []
+            values = []
+            energies = dict()
+            for val in val_in.strip().split('\n'):
+                val = val.strip()
+                if val[0].isalpha():
+                    keys = [k.strip() for k in energy_keys_re.findall(val)]
+                    keys = ['P%s' % k if k.startswith('res') else k for k in keys if k]
+                else:
+                    values = val.split()
+                    for n, key in enumerate(keys):
+
+                        if key == 'Temperature':
+                            energies[key] = float(values[n]) * ureg.kelvin
+                        elif key.startswith('Pres'):
+                            key = key.rstrip(' (bar)')
+                            energies[key] = float(values[n]) * ureg.bar
+                        else:
+                            energies[key] = float(values[n]) / MOL * ureg.kJ
+            return energies
+
+        def str_to_step_info(val_in):
+            val = val_in.strip().split('\n')
+            keys = val[0].split()
+            values = [float(v) for v in val[1].split()]
+            return {key: values[n] for n, key in enumerate(keys)}
+
+        thermo_quantities = [
+            Quantity(
+                'energies',
+                r'Energies \(kJ/mol\)\s*([\s\S]+?)\n\n',
+                str_operation=str_to_energies, convert=False),
+            Quantity(
+                'step_info',
+                r'(Step.+\n[\d\.\- ]+)',
+                str_operation=str_to_step_info, convert=False)]
+
+        self._quantities = [
+            Quantity('time_start', r'Log file opened on (.+)', flatten=False),
+            Quantity(
+                'host_info',
+                r'Host:\s*(\S+)\s*pid:\s*(\d+)\s*rank ID:\s*(\d+)\s*number of ranks:\s*(\d*)'),
+            Quantity('module_version', r'GROMACS:\s*(.+?),\s*VERSION\s*(\S+)', flatten=False),
+            Quantity('execution_path', r'Executable:\s*(.+)'),
+            Quantity('working_path', r'Data prefix:\s*(.+)'),
+            # TODO cannot understand treatment of the command line in the old parser
+            Quantity(
+                'header',
+                r'(GROMACS version:[\s\S]+?)\n\n', str_operation=str_to_header),
+            Quantity(
+                'input_parameters',
+                r'Input Parameters:\s*([\s\S]+?)\n\n', str_operation=str_to_input_parameters),
+            Quantity(
+                'step',
+                r'(Step\s*Time[\s\S]+?Energies[\s\S]+?\n\n)',
+                repeats=True, sub_parser=TextParser(quantities=thermo_quantities)),
+            Quantity(
+                'averages',
+                r'A V E R A G E S  ====>([\s\S]+?\n\n\n)',
+                sub_parser=TextParser(quantities=thermo_quantities)),
+            Quantity('time_end', r'Finished \S+ on rank \d+ (.+)', flatten=False)]
 
     def get_pbc(self):
-        pbc = self.get('pbc', 'xyz')
+        pbc = self.get('input_parameters', {}).get('pbc', 'xyz')
         return ['x' in pbc, 'y' in pbc, 'z' in pbc]
 
     def get_sampling_settings(self):
-        integrator = self.get('integrator', 'md').lower()
+        input_parameters = self.get('input_parameters', {})
+        integrator = input_parameters.get('integrator', 'md').lower()
         if integrator in ['l-bfgs', 'cg', 'steep']:
             sampling_method = 'geometry_optimization'
         elif integrator in ['bd']:
@@ -208,10 +146,10 @@ class GromacsLogParser(TextParser):
             sampling_method = 'molecular_dynamics'
 
         ensemble_type = 'NVE' if sampling_method == 'molecular_dynamics' else None
-        tcoupl = self.get('tcoupl', 'no').lower()
+        tcoupl = input_parameters.get('tcoupl', 'no').lower()
         if tcoupl != 'no':
             ensemble_type = 'NVT'
-            pcoupl = self.get('pcoupl', 'no').lower()
+            pcoupl = input_parameters.get('pcoupl', 'no').lower()
             if pcoupl != 'no':
                 ensemble_type = 'NPT'
 
@@ -220,30 +158,31 @@ class GromacsLogParser(TextParser):
             ensemble_type=ensemble_type)
 
     def get_tpstat_settings(self):
-        target_T = self.get('ref-t', 0, unit='K')
+        input_parameters = self.get('input_parameters', {})
+        target_T = input_parameters.get('ref-t', 0) * ureg.kelvin
 
         thermostat_type = None
-        tcoupl = self.get('tcoupl', 'no').lower()
+        tcoupl = input_parameters.get('tcoupl', 'no').lower()
         if tcoupl != 'no':
             thermostat_type = 'Velocity Rescaling' if tcoupl == 'v-rescale' else tcoupl.title()
 
-        thermostat_tau = self.get('tau-t', 0, unit='ps')
+        thermostat_tau = input_parameters.get('tau-t', 0) * ureg.ps
 
         # TODO infer langevin_gamma [s] from bd_fric
         # bd_fric = self.get('bd-fric', 0, unit='amu/ps')
         langevin_gamma = None
 
-        target_P = self.get('ref-p', 0, unit='bar')
+        target_P = input_parameters.get('ref-p', 0) * ureg.bar
         # if P is array e.g. for non-isotropic pressures, get average since metainfo is float
         if hasattr(target_P, 'shape'):
             target_P = np.average(target_P)
 
         barostat_type = None
-        pcoupl = self.get('pcoupl', 'no').lower()
+        pcoupl = input_parameters.get('pcoupl', 'no').lower()
         if pcoupl != 'no':
             barostat_type = pcoupl.title()
 
-        barostat_tau = self.get('tau-p', 0, unit='ps')
+        barostat_tau = input_parameters.get('tau-p', 0) * ureg.ps
 
         return dict(
             target_T=target_T, thermostat_type=thermostat_type, thermostat_tau=thermostat_tau,
@@ -267,11 +206,17 @@ class GromacsEDRParser(FileParser):
     @property
     def fileedr(self):
         if self._file_handler is None:
-            self._file_handler = panedr.edr_to_df(self.mainfile)
+            try:
+                self._file_handler = panedr.edr_to_df(self.mainfile)
+            except Exception:
+                self.logger.error('Error reading edr file.')
 
         return self._file_handler
 
     def parse(self, key):
+        if self.fileedr is None:
+            return
+
         val = self.fileedr.get(key, None)
         if self._results is None:
             self._results = dict()
@@ -279,20 +224,21 @@ class GromacsEDRParser(FileParser):
         if val is not None:
             val = np.asarray(val)
         if key in self._energy_keys:
-            val = pint.Quantity(val / MOL, 'kJ')
+            val = val / MOL * ureg.kJ
         elif key in self._temperature_keys:
-            val = pint.Quantity(val, 'K')
+            val = val * ureg.kelvin
         elif key in self._pressure_keys:
-            val = pint.Quantity(val, 'bar')
+            val = val * ureg.bar
         elif key in self._time_keys:
-            val = pint.Quantity(val, 'ps')
+            val = val * ureg.ps
 
         self._results[key] = val
 
-    def get_keys(self):
+    def keys(self):
         return list(self.fileedr.keys())
 
-    def get_length(self):
+    @property
+    def length(self):
         return self.fileedr.shape[0]
 
 
@@ -352,17 +298,23 @@ class MDAnalysisParser(FileParser):
     @property
     def universe(self):
         if self._file_handler is None:
-            args = [f for f in [self.trajectory_file] if f is not None]
-            self._file_handler = MDAnalysis.Universe(self.mainfile, *args)
+            try:
+                args = [f for f in [self.trajectory_file] if f is not None]
+                self._file_handler = MDAnalysis.Universe(self.mainfile, *args)
+            except Exception:
+                self.logger.error('Error setting up MDAnalysis.')
         return self._file_handler
 
     def parse(self, key):
-        if not MDAnalysis:
-            return
-
         if self._results is None:
             self._results = dict()
 
+        if self.universe is None:
+            return
+
+        if not MDAnalysis:
+            return
+
         atoms = list(self.universe.atoms)
         try:
             trajectory = self.universe.trajectory
@@ -373,7 +325,7 @@ class MDAnalysisParser(FileParser):
         val = None
         if key == 'timestep':
             val = trajectory.dt
-            unit = 'ps'
+            unit = ureg.ps
         elif key == 'atom_labels':
             val = [
                 MDAnalysis.topology.guessers.guess_atom_element(atom.name)
@@ -384,22 +336,22 @@ class MDAnalysisParser(FileParser):
             val = len(trajectory)
         elif key == 'positions':
             val = [traj.positions if traj.has_positions else None for traj in trajectory]
-            unit = 'angstrom'
+            unit = ureg.angstrom
         elif key == 'velocities':
             val = [traj.velocities if traj.has_velocities else None for traj in trajectory]
-            unit = 'angstrom/ps'
+            unit = ureg.angstrom / ureg.ps
         elif key == 'forces':
             val = [traj.forces / MOL if traj.has_forces else None for traj in trajectory]
-            unit = 'kJ/angstrom'
+            unit = ureg.kJ / ureg.angstrom
         elif key == 'cell':
             val = [traj.triclinic_dimensions for traj in trajectory]
-            unit = 'angstrom'
+            unit = ureg.angstrom
 
         if unit is not None:
             if isinstance(val, list):
-                val = [pint.Quantity(v, unit) if v is not None else v for v in val]
+                val = [v * unit if v is not None else v for v in val]
             else:
-                val = pint.Quantity(val, unit)
+                val = val * unit
 
         self._results[key] = val
 
@@ -413,6 +365,9 @@ class GromacsParser(FairdiParser):
         self.log_parser = GromacsLogParser()
         self.traj_parser = MDAnalysisParser()
         self.energy_parser = GromacsEDRParser()
+        self._metainfo_mapping = {
+            'LJ (SR)': 'Leonard-Jones', 'Coulomb (SR)': 'coulomb',
+            'Potential': 'potential', 'Kinetic En.': 'kinetic'}
 
     def get_gromacs_file(self, ext):
         files = [d for d in self._gromacs_files if d.endswith(ext)]
@@ -423,7 +378,11 @@ class GromacsParser(FairdiParser):
         # we assume that the file has the same basename as the log file e.g.
         # out.log would correspond to out.tpr and out.trr and out.edr
         for f in files:
-            if f.split('.')[0] == self._basename:
+            if f.rsplit('.', 1)[0] == self._basename:
+                return os.path.join(self._maindir, f)
+
+        for f in files:
+            if f.rsplit('.', 1)[0].startswith(self._basename):
                 return os.path.join(self._maindir, f)
 
         # if the files are all named differently, we guess that the one that does not
@@ -435,7 +394,7 @@ class GromacsParser(FairdiParser):
         for f in files:
             count = 0
             for reff in self._gromacs_files:
-                if f.split('.')[0] == reff.split('.')[0]:
+                if f.rsplit('.', 1)[0] == reff.rsplit('.', 1)[0]:
                     count += 1
             if count == 1:
                 return os.path.join(self._maindir, f)
@@ -445,42 +404,63 @@ class GromacsParser(FairdiParser):
 
     def parse_thermodynamic_data(self):
         sec_run = self.archive.section_run[-1]
-        sec_sccs = sec_run.section_single_configuration_calculation
 
-        energy_keys_mapping = {
-            'LJ (SR)': 'Leonard-Jones', 'Coulomb (SR)': 'coulomb',
-            'Potential': 'potential', 'Kinetic En.': 'kinetic'}
+        forces = self.traj_parser.get('forces')
+        for n, forces_n in enumerate(forces):
+            sec_scc = sec_run.m_create(SingleConfigurationCalculation)
+            sec_scc.atom_forces = forces_n
+            sec_scc.single_configuration_calculation_to_system_ref = sec_run.section_system[n]
+            sec_scc.single_configuration_to_calculation_method_ref = sec_run.section_method[-1]
 
-        n_evaluations = self.energy_parser.get_length()
-        create_scc = True
-        if sec_sccs:
-            if len(sec_sccs) != n_evaluations:
-                self.logger.warn(
-                    '''Mismatch in number of calculations and number of thermodynamic
-                    evaluations, will create new sections''',
-                    data=dict(n_calculations=len(sec_sccs), n_evaluations=n_evaluations))
+        # try to get it from log file
+        steps = self.log_parser.get('step', [])
+        if self.log_parser.get('averages') is not None:
+            steps.append(self.log_parser.get('averages'))
+        thermo_data = dict()
+        for step in steps:
+            if step.energies is None:
+                continue
+            keys = step.energies.keys()
+            for key in keys:
+                thermo_data.setdefault(key, [])
+                thermo_data[key].append(step.energies.get(key))
+            info = step.get('step_info', {})
+            thermo_data.setdefault('Time', [])
+            thermo_data['Time'].append(info.get('Time', None))
+        n_evaluations = len(steps)
 
-            else:
-                create_scc = False
+        if not thermo_data:
+            # get it from edr file
+            thermo_data = self.energy_parser
+            n_evaluations = self.energy_parser.length
 
-        timestep = self.traj_parser.get('timestep', 1.0, unit='ps')
+        create_scc = False
+        if len(forces) != n_evaluations:
+            self.logger.warn(
+                'Mismatch in number of calculations and number of thermodynamic'
+                'evaluations, will create new sections')
+            create_scc = True
+
+        timestep = self.traj_parser.get('timestep')
+        if timestep is None:
+            timestep = self.log_parser.get('input_parameters', {}).get('dt', 1.0) * ureg.ps
+
+        # TODO add other energy contributions, properties
+        energy_keys = ['LJ (SR)', 'Coulomb (SR)', 'Potential', 'Kinetic En.']
 
-        keys = self.energy_parser.get_keys()
         for n in range(n_evaluations):
             if create_scc:
                 sec_scc = sec_run.m_create(SingleConfigurationCalculation)
             else:
-                sec_scc = sec_sccs[n]
+                sec_scc = sec_run.section_single_configuration_calculation[n]
 
-            for key in keys:
-                val = self.energy_parser.get(key)[n]
-                if key in energy_keys_mapping:
-                    sec_energy = sec_scc.m_create(EnergyContribution)
-                    sec_energy.energy_contibution_kind = energy_keys_mapping[key]
-                    sec_energy.energy_contribution_value = val
+            for key in thermo_data.keys():
+                val = thermo_data.get(key)[n]
+                if val is None:
+                    continue
 
-                elif key == 'Total Energy':
-                    sec_scc.energy_method_current = val
+                if key == 'Total Energy':
+                    sec_scc.energy_total = val
 
                 elif key == 'Pressure':
                     sec_scc.pressure = val
@@ -490,6 +470,10 @@ class GromacsParser(FairdiParser):
 
                 elif key == 'Time':
                     sec_scc.time_step = int((val / timestep).magnitude)
+                if key in energy_keys:
+                    sec_energy = sec_scc.m_create(EnergyContribution)
+                    sec_energy.energy_contribution_kind = self._metainfo_mapping[key]
+                    sec_energy.energy_contribution_value = val
 
     def parse_topology(self):
         sec_run = self.archive.section_run[-1]
@@ -512,23 +496,13 @@ class GromacsParser(FairdiParser):
             sec_interaction.interaction_kind = interaction[0]
             sec_interaction.interaction_parameters = interaction[1]
 
+        sec_run.section_method[-1].method_to_topology_ref = sec_topology
+
     def parse_system(self):
         sec_run = self.archive.section_run[-1]
 
         n_frames = self.traj_parser.get('n_frames', 0)
 
-        create_scc = True
-        sec_sccs = sec_run.section_single_configuration_calculation
-        if sec_sccs:
-            if len(sec_sccs) != n_frames:
-                self.logger.warn(
-                    '''Mismatch in number of calculations and number of structures,
-                    will create new sections''',
-                    data=dict(n_calculations=len(sec_sccs), n_structures=n_frames))
-
-            else:
-                create_scc = False
-
         pbc = self.log_parser.get_pbc()
         for n in range(n_frames):
             positions = self.traj_parser.get_positions(n)
@@ -539,6 +513,7 @@ class GromacsParser(FairdiParser):
             sec_system.number_of_atoms = self.traj_parser.get_n_atoms(n)
             sec_system.configuration_periodic_dimensions = pbc
             sec_system.simulation_cell = self.traj_parser.get_cell(n)
+            sec_system.lattice_vectors = self.traj_parser.get_cell(n)
             sec_system.atom_labels = self.traj_parser.get_atom_labels(n)
             sec_system.atom_positions = positions
 
@@ -546,13 +521,9 @@ class GromacsParser(FairdiParser):
             if velocities is not None:
                 sec_system.atom_velocities = velocities
 
-            forces = self.traj_parser.get_forces(n)
-            if forces is not None:
-                if create_scc:
-                    sec_scc = sec_run.m_create(SingleConfigurationCalculation)
-                else:
-                    sec_scc = sec_sccs[n]
-                sec_scc.atom_forces = forces
+    def parse_method(self):
+        self.archive.section_run[-1].m_create(Method)
+        # TODO what to put here
 
     def parse_sampling_method(self):
         sec_run = self.archive.section_run[-1]
@@ -564,10 +535,11 @@ class GromacsParser(FairdiParser):
         sec_sampling_method.ensemble_type = sampling_settings['ensemble_type']
         sec_sampling_method.x_gromacs_integrator_type = sampling_settings['integrator_type']
 
-        timestep = self.log_parser.get('dt', 0)
+        input_parameters = self.log_parser.get('input_parameters', {})
+        timestep = input_parameters.get('dt', 0)
         sec_sampling_method.x_gromacs_integrator_dt = timestep
 
-        nsteps = self.log_parser.get('nsteps', 0)
+        nsteps = input_parameters.get('nsteps', 0)
         sec_sampling_method.x_gromacs_number_of_steps_requested = nsteps
 
         tp_settings = self.log_parser.get_tpstat_settings()
@@ -605,14 +577,13 @@ class GromacsParser(FairdiParser):
         sec_input_output_files.x_gromacs_inout_file_eneredr = edr_file
 
         sec_control_parameters = sec_run.m_create(x_gromacs_section_control_parameters)
-        keys = self.log_parser.keys()
-        for key in keys:
-            val = self.log_parser.get(key)
-            if val is None:
-                continue
-            key = 'x_gromacs_inout_control_%s' % key.replace('-', '').lower()
+        input_parameters = self.log_parser.get('input_parameters', {})
+        input_parameters.update(self.log_parser.get('header', {}))
+        for key, val in input_parameters.items():
+            key = 'x_gromacs_inout_control_%s' % key.replace('-', '').replace(' ', '_').lower()
             if hasattr(sec_control_parameters, key):
-                setattr(sec_control_parameters, key, str(val))
+                val = str(val) if not isinstance(val, np.ndarray) else val
+                setattr(sec_control_parameters, key, val)
 
     def init_parser(self):
         self.log_parser.mainfile = self.filepath
@@ -626,22 +597,39 @@ class GromacsParser(FairdiParser):
     def parse(self, filepath, archive, logger):
         self.filepath = os.path.abspath(filepath)
         self.archive = archive
-        self.logger = logging if logger is None else logger
+        self.logger = logging.getLogger(__name__) if logger is None else logger
         self._maindir = os.path.dirname(self.filepath)
         self._gromacs_files = os.listdir(self._maindir)
-        self._basename = os.path.basename(filepath).split('.')[0]
+        self._basename = os.path.basename(filepath).rsplit('.', 1)[0]
 
         self.init_parser()
 
         sec_run = self.archive.m_create(Run)
 
-        version = self.log_parser.get('version', ['VERSION', 'unknown'])
         sec_run.program_name = 'GROMACS'
-        sec_run.program_version = version[1]
+
+        header = self.log_parser.get('header', {})
+        sec_run.program_version = header.get('GROMACS version', 'unknown').lstrip('VERSION ')
+
+        for key in ['start', 'end']:
+            time = self.log_parser.get('time_%s')
+            if time is None:
+                continue
+            setattr(sec_run, 'time_run_date_%s' % key, datetime.datetime.strptime(
+                time, '%a %b %d %H:%M:%S %Y').timestamp())
+
+        host_info = self.log_parser.get('host_info')
+        if host_info is not None:
+            sec_run.x_gromacs_program_execution_host = host_info[0]
+            sec_run.x_gromacs_parallel_task_nr = host_info[1]
+            sec_run.x_gromacs_number_of_tasks = host_info[2]
+
+        self.parse_method()
 
         self.parse_sampling_method()
 
         topology_file = self.get_gromacs_file('tpr')
+        # I have no idea if output trajectory file can be specified in input
         trajectory_file = self.get_gromacs_file('trr')
         self.traj_parser.mainfile = topology_file
         self.traj_parser.trajectory_file = trajectory_file
@@ -650,6 +638,7 @@ class GromacsParser(FairdiParser):
 
         self.parse_system()
 
+        # TODO read also from ene
         edr_file = self.get_gromacs_file('edr')
         self.energy_parser.mainfile = edr_file
 
