@@ -29,6 +29,7 @@ respective file format. They both use separate file (:class:`RunFileParser`) and
 (:class:`OutcarTextParser`) parsers to read content content from either xml or text files.
 '''
 
+from typing import List
 import os
 import numpy as np
 import logging
@@ -195,28 +196,28 @@ class OutcarTextParser(TextParser):
 
         scf_iteration = [
             Quantity(
-                'energy_total', r'free energy\s*TOTEN\s*=\s*([\d\.]+)\s*eV',
+                'energy_total', r'free energy\s*TOTEN\s*=\s*([\d\.\-]+)\s*eV',
                 repeats=False, dtype=float),
             Quantity(
-                'energy_entropy0', r'energy without entropy\s*=\s*([\d\.]+)',
+                'energy_entropy0', r'energy without entropy\s*=\s*([\d\.\-]+)',
                 repeats=False, dtype=float),
             Quantity(
-                'energy_T0', r'energy\(sigma\->0\)\s*=\s*([\d\.]+)',
+                'energy_T0', r'energy\(sigma\->0\)\s*=\s*([\d\.\-]+)',
                 repeats=False, dtype=float),
             Quantity(
                 'energy_components',
-                r'Free energy of the ion-electron system \(eV\)\s*\-+([\s\S]+?)\-{100}',
+                r'Free energy of the ion-electron system \(eV\)\s*\-+([\s\S]+?)\-{10}',
                 str_operation=get_key_values, convert=False)
         ]
 
         self._quantities.append(Quantity(
             'calculation',
-            r'(\-\-\s*Iteration\s*\d+\(\s*1\)\s*[\s\S]+?(?:FREE ENERGIE OF THE ION\-ELECTRON SYSTEM \(eV\)))'
+            r'(\-\-\s*Iteration\s*\d+\(\s*\d+\s*\)\s*[\s\S]+?(?:FREE ENERGIE OF THE ION\-ELECTRON SYSTEM \(eV\)))'
             r'([\s\S]+?\-{100})',
             repeats=True, sub_parser=TextParser(quantities=[
                 Quantity(
                     'scf_iteration',
-                    r'Iteration\s*\d+\(\s*\d+\)([\s\S]+?energy\(sigma\->0\)\s*=\s*[\d\.]+)',
+                    r'Iteration\s*\d+\(\s*\d+\s*\)([\s\S]+?energy\(sigma\->0\)\s*=\s*.+)',
                     repeats=True, sub_parser=TextParser(quantities=scf_iteration)),
                 Quantity(
                     'energies',
@@ -273,7 +274,7 @@ class OutcarTextParser(TextParser):
             str_operation=get_key_values, repeats=False, convert=False))
 
         self._quantities.append(Quantity(
-            'ions_per_type', r'ions per type =([ \d]+)', dtype=int, repeats=False))
+            'ions_per_type', r'ions per type =\s*([ \d]+)', dtype=int, repeats=False))
 
         self._quantities.append(Quantity(
             'species', r'TITEL\s*=\s*(\w+) ([A-Z][a-z]*)', dtype=str, repeats=True))
@@ -424,6 +425,7 @@ class OutcarContentParser(ContentParser):
                             ions = [int(n) for n in line.split()]
                         except Exception:
                             pass
+            ions = [i for i in ions if not isinstance(i, str)]
             if len(ions) != len(species):
                 self.parser.logger.error('Inconsistent number of ions and species.')
                 return self._atom_info
@@ -477,7 +479,9 @@ class OutcarContentParser(ContentParser):
             section = self.parser.get('calculation', [{}] * (
                 n_calc + 1))[n_calc].get('scf_iteration', [{}] * (n_scf + 1))[n_scf]
         for key in ['energy_total', 'energy_T0', 'energy_entropy0']:
-            energies[key] = section.get(key) * multiplier
+            val = section.get(key)
+            if val is not None:
+                energies[key] = val * multiplier
 
         energies.update(section.get('energy_components', {}))
         return energies
@@ -606,32 +610,6 @@ class OutcarContentParser(ContentParser):
         return dos, fields
 
 
-# # TODO
-# This was not used in the last commit. It was also not working, because it applies regex to bytes array.
-# The xml.sax parser can parse XML that contains the mentioned entities. See test_broken_xml.
-# I am not sure if they still need to bo removed to avoid bad values. If this needs to be
-# added somewhere, overwrite FileParser.open might be a good place
-# class XMLParser(ElementTree.XMLParser):
-
-#     rx = re.compile("&#([0-9]+);|&#x([0-9a-fA-F]+);")
-
-#     def feed(self, data):
-#         m = self.rx.search(data)
-#         if m is not None:
-#             target = m.group(1)
-#             if target:
-#                 num = int(target)
-#             else:
-#                 num = int(m.group(2), 16)
-#             if not(num in (0x9, 0xA, 0xD) or 0x20 <= num <= 0xD7FF or 0xE000 <= num <= 0xFFFD or 0x10000 <= num <= 0x10FFFF):
-#                 # is invalid xml character, cut it out of the stream
-#                 mstart, mend = m.span()
-#                 mydata = data[:mstart] + data[mend:]
-#         else:
-#             mydata = data
-#         super().feed(mydata)
-
-
 class RunXmlContentHandler(ContentHandler):
     def __init__(self):
         self._text = ''
@@ -715,7 +693,7 @@ class RunFileParser(FileParser):
         except Exception as e:
             # support broken XML structure
             if self.logger:
-                self.logger.error('could not parse all xml', exc_info=e)
+                self.logger.warning('could not parse all xml', exc_info=e)
             content_handler.clear_stack()
 
         self._results = content_handler
@@ -747,6 +725,11 @@ class RunContentParser(ContentParser):
         self.parser.parse()
 
     def _get_key_values(self, path, repeats=False, array=False):
+        def parse_float_str_vector(str_vector: List[str]):
+            return [
+                'nan' if '*' in x else x
+                for x in str_vector]
+
         root, base_name = path.strip('/').rsplit('/', 1)
 
         attrib = re.search(self._re_attrib, base_name)
@@ -772,7 +755,9 @@ class RunContentParser(ContentParser):
 
         result = dict()
         if array:
-            value = [d[0].split() for d in data if d[0]]
+            value = [
+                parse_float_str_vector(item[0].split())
+                for item in data if item[0]]
             value = [d[0] if len(d) == 1 and not repeats else d for d in value]
             dtype = data[0][2]
             result[base_name] = np.array(value, dtype=self._dtypes.get(dtype, float))
@@ -790,7 +775,7 @@ class RunContentParser(ContentParser):
                     value = [v == 'T' for v in value]
                 if dtype == float:
                     # prevent nan printouts
-                    value = ['nan' if '*' in v else v for v in value]
+                    value = parse_float_str_vector(value)
                 # using numpy array does not work
                 value = convert(value, dtype)
                 value = value[0] if len(value) == 1 else value
@@ -1061,7 +1046,7 @@ class VASPParser(FairdiParser):
                 r'?\s*<modeling>'
                 r'?\s*<generator>'
                 r'?\s*<i name="program" type="string">\s*vasp\s*</i>'
-                r'?|^\svasp[\.\d]+\s*\w+\s*\(build'),
+                r'?|^\svasp[\.\d]+.+?\s*\(build'),
             supported_compressions=['gz', 'bz2', 'xz'], mainfile_alternative=True)
 
         self._metainfo_env = m_env
@@ -1192,12 +1177,12 @@ class VASPParser(FairdiParser):
 
             energies = self.parser.get_energies(n_calc, n_scf)
             for key, val in energies.items():
-                if val is None:
-                    continue
-                val = pint.Quantity(val, 'eV')
                 metainfo_key = self.parser.metainfo_mapping.get(key, None)
-                if metainfo_key is None:
+                if val is None or metainfo_key is None:
                     continue
+
+                val = pint.Quantity(val, 'eV')
+
                 try:
                     setattr(section, '%s%s' % (metainfo_key, ext), val)
                 except Exception:
