@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 '''
 This module contains functionality to use old 'legacy' NOMAD CoE parsers with the
@@ -367,6 +371,7 @@ class LegacyParser(MatchingParser):
         parser_class_name = self.parser_class_name.split('.')[-1]
         self.__parser_impl = module_name, parser_class_name
         self.__parser_class = None
+        self._metainfo_env = None
 
     @property
     def metainfo_env(self):
@@ -375,7 +380,7 @@ class LegacyParser(MatchingParser):
             module = importlib.import_module('.'.join(module_name + ['metainfo']))
             self._metainfo_env = getattr(module, 'm_env')
 
-        return super().metainfo_env
+        return self._metainfo_env
 
     @property
     def parser_class(self):
